local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format("Module '%s' requires AshranBuddy", "Event Timer"))
end

local EventTimer = AshranBuddy:RegisterModule("EventTimer")
LibStub("AceTimer-3.0"):Embed(EventTimer)

-- Inherited
function EventTimer:OnEnable()
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "OnChatMessageMonsterEmote")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChangedNewArea")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	
	self:CreateContainerFrame(220, 65)
	
	if (AshranBuddy.db.profile.locked == false) then
		EventTimer:Unlock()
	end
end

-- Inherited
function EventTimer:OnDisable()
	EventTimer:UnregisterAllEvents()
end

function EventTimer:OnChatMessageMonsterEmote(event, message, sender, language, channelString, target, flags, unused1, channelNumber, channelName, unused2, counter)
	if (AshranBuddy:IsInAshran()) then
		for i = 1, #self.statusBars do
			local statusBar = self.statusBars[i]
			if (message:find(statusBar.eventId)) then
				self:OnAshranEventAnnounced(statusBar)
			end
		end
	end
end

function EventTimer:OnZoneChangedNewArea(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
	else
		if (AshranBuddy.db.profile.locked == false) then
			EventTimer:Unlock()
		else
			self.containerFrame:Hide()
		end
	end
end

function EventTimer:OnPlayerEnteringWorld(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
	else
		if (AshranBuddy.db.profile.locked == false) then
			EventTimer:Unlock()
		else
			self.containerFrame:Hide()
		end
	end
end

function EventTimer:OnUpdate()
	if (AshranBuddy:IsInAshran()) then
		for i = 1, #EventTimer.statusBars do
			local statusBar = EventTimer.statusBars[i]
			local value = statusBar:GetValue()
			local minValue, maxValue = statusBar:GetMinMaxValues()
			local exists = EventTimer:FindWorldMapLandmark(statusBar.eventName)

			if (exists) then
				EventTimer:OnAshranEventStarted(statusBar)
			end
			if (not exists and value == maxValue) then
				EventTimer:OnAshranEventEnded(statusBar)
			end
		end
	end
end

function EventTimer:OnMouseDown()
	if (AshranBuddy.db.profile.locked == false) then
		self:StartMoving()
	end
end

function EventTimer:OnMouseUp()
	if (AshranBuddy.db.profile.locked == false) then
		self:StopMovingOrSizing()
	end
end

function EventTimer:OnStatusBarClick(button)
	if (button == "LeftButton" and IsShiftKeyDown() and IsInRaid()) then
		local value = self:GetValue()
		local minValue, maxValue = self:GetMinMaxValues()
		local timeLeft = maxValue - value
		
		local message = ""
		
		if (timeLeft == 0) then
			message = "AshranBuddy: [Event: " .. self.eventName .. "] has begun."
		end
		if (timeLeft > 0) then
			message = "AshranBuddy: [Event: " .. self.eventName .. "] will begin in " .. string.format("%.2d:%.2d", timeLeft/60%60, timeLeft%60)
		end
		
		SendChatMessage(message, "RAID")
	end
end

function EventTimer:OnAshranEventAnnounced(statusBar)
	if (AshranBuddy:IsInAshran()) then
		statusBar:SetValue(0)
		statusBar:SetStatusBarColor(0.26, 0.42, 1)
		statusBar:Show()
		
		statusBar.eventTimer = EventTimer:ScheduleRepeatingTimer("OnTimerCallback", 1, statusBar)
	end
end

function EventTimer:OnTimerCallback(statusBar)
	local value = statusBar:GetValue()
	local minValue, maxValue = statusBar:GetMinMaxValues()
	local timeLeft = maxValue - value
	
	statusBar:SetEventTime(string.format("%.2d:%.2d", timeLeft/60%60, timeLeft%60))
	statusBar:SetValue(value + 1)

	if (value >= maxValue) then
		self:CancelTimer(statusBar.eventTimer)
		statusBar.eventTimer = nil
		if (statusBar.eventName == "Ogre Fires") then
			-- TODO: Ogre Fires cannot be tracked correctly due to a Blizzard issue
			statusBar:Hide()
		else
			self:OnAshranEventStarted(statusBar)
		end
	end
end

function EventTimer:OnAshranEventStarted(statusBar)
	if (AshranBuddy:IsInAshran()) then
		local minValue, maxValue = statusBar:GetMinMaxValues()
		
		statusBar:SetEventTime("")
		statusBar:SetValue(maxValue)
		statusBar:SetStatusBarColor(0, 1, 0)
		statusBar:Show()
	end
end

function EventTimer:OnAshranEventEnded(statusBar)
	if (AshranBuddy:IsInAshran()) then
		statusBar:Hide()
	end
end

function EventTimer:Lock()
	if (not AshranBuddy:IsInAshran()) then
		self.containerFrame:Hide()
	end
	self.handle:Hide()
	
	local point, relativeTo, relativePoint, x, y = self.containerFrame:GetPoint(1)
	AshranBuddy.db.profile.point = point
	AshranBuddy.db.profile.relativePoint = relativePoint
	AshranBuddy.db.profile.x = x
	AshranBuddy.db.profile.y = y
end

function EventTimer:Unlock()
	self.containerFrame:Show()
	self.handle:Show()
end

function EventTimer:CreateContainerFrame(width, height)
	self.statusBars = { }
	
	local profile = AshranBuddy.db.profile
	
	local frame = self.containerFrame
	frame:SetParent(UIParent)
	frame:SetSize(width, height)
	frame:SetPoint(profile.point, UIParent, profile.relativePoint, profile.x, profile.y)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetBackdrop({ bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background" })
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetToplevel(true)
	
	local handle = CreateFrame("Frame", nil, frame)
	handle:SetSize(width, height)
	handle:SetPoint("TOPLEFT", frame)
	handle:Hide()

	local handleBox = handle:CreateTexture(nil, "OVERLAY", nil, -1)
	handleBox:SetTexture(0, 0, 0, 0.6)
	handleBox:SetSize(width, height)
	handleBox:SetPoint("TOPLEFT")
	
	local handleFontString = handle:CreateFontString("HandleLabel", "OVERLAY", "TextStatusBarText")
	handleFontString:SetPoint("CENTER")
	handleFontString:SetText("Ashran Buddy\n\nYou MUST Type /ab lock or\n/ashranbuddy lock to activate.\nBars won't show until event is announced")

	-- Children
	self.handle = handle
	self.statusBars[1] = self:CreateStatusBarFrame("178019", "EmpoweredOre", "Collect Empowered Ore", frame, -5)
	self.statusBars[2] = self:CreateStatusBarFrame("178021", "OgreFires", "Ogre Fires", frame, -20)
	self.statusBars[3] = self:CreateStatusBarFrame("178020", "RisenSpirits", "Risen Spirits", frame, -35)
	self.statusBars[4] = self:CreateStatusBarFrame("178022", "StadiumRacing", "Stadium Racing", frame, -50)

	-- Events
	frame:SetScript("OnUpdate", EventTimer.OnUpdate)
	frame:SetScript("OnMouseDown", EventTimer.OnMouseDown)
	frame:SetScript("OnMouseUp", EventTimer.OnMouseUp)
end

function EventTimer:CreateStatusBarFrame(id, name, eventName, parent, y)
	local statusBar = CreateFrame("StatusBar", name, parent)
	statusBar:SetWidth(200)
	statusBar:SetHeight(10)
	statusBar:SetPoint("TOPLEFT", parent, "TOPLEFT", 5, y)
	statusBar:SetMinMaxValues(0, 180)
	statusBar:SetValue(0)
	statusBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
	statusBar:SetStatusBarColor(0.26, 0.42, 1)

	-- Artwork Layer
	local borderLeft = statusBar:CreateTexture("BorderLeft", "ARTWORK")
	borderLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderLeft:SetTexCoord(0.007843, 0.043137, 0.193548, 0.774193)
	borderLeft:SetPoint("LEFT", -3, 0)
	borderLeft:SetSize(9, 14)

	local borderRight = statusBar:CreateTexture("BorderRight", "ARTWORK")
	borderRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderRight:SetTexCoord(0.043137, 0.007843, 0.193548, 0.774193)
	borderRight:SetPoint("RIGHT", 3, 0)
	borderRight:SetSize(9, 14)
	
	local borderMid = statusBar:CreateTexture("BorderMid", "ARTWORK")
	borderMid:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderMid:SetTexCoord(0.113726, 0.1490196, 0.193548, 0.774193)
	borderMid:SetPoint("TOPLEFT", borderLeft, "TOPRIGHT")
	borderMid:SetPoint("BOTTOMRIGHT", borderRight, "BOTTOMLEFT")
	borderMid:SetSize(9, 14)

	local eventNameFontString = statusBar:CreateFontString("EventNameLabel", "ARTWORK", "TextStatusBarText")
	eventNameFontString:SetPoint("LEFT")

	local eventTimeFontString = statusBar:CreateFontString("EventTimeLabel", "ARTWORK", "TextStatusBarText")
	eventTimeFontString:SetPoint("RIGHT")
	
	-- Background Layer
	local barFull = statusBar:CreateTexture("BarFull", "BACKGROUND")
	barFull:SetVertexColor(0.04, 0.07, 0.18)
	
	statusBar.eventId = id
	statusBar.eventName = eventName
	statusBar.eventNameFontString = eventNameFontString
	statusBar.eventTimeFontString = eventTimeFontString
	statusBar.mouseDown = false
	
	statusBar.GetEventId = function(self)
		return self.eventId
	end
	statusBar.GetEventName = function(self)
		return self.eventName
	end
	statusBar.GetEventTitle = function(self)
		return self.eventNameFontString:GetText()
	end
	statusBar.SetEventTitle = function(self, value)
		self.eventNameFontString:SetText(value)
	end
	statusBar.GetEventTime = function(self)
		return self.eventTimeFontString:GetText()
	end
	statusBar.SetEventTime = function(self, value)
		self.eventTimeFontString:SetText(value)
	end

	statusBar:SetScript("OnMouseDown", EventTimer.OnStatusBarClick)
	
	statusBar:SetEventTitle(eventName)
	statusBar:SetEventTime("")
	statusBar:Hide()
	
	return statusBar
end

function EventTimer:FindWorldMapLandmark(eventName) 
	local count = GetNumMapLandmarks()
	for i = 1, count do
        local worldMapPOIName = "WorldMapFramePOI" .. i
        local worldMapPOI = _G[worldMapPOIName]
		
		local name, description, textureIndex, x, y, mapLinkID, inBattleMap, graveyardID, areaID, poiID, isObjectIcon = GetMapLandmarkInfo(i)
		if (name == eventName) then
			return true
		end
	end
	return false
end

-- Inherited
function EventTimer:OnChatCommand(input)
	if (input == "unlock") then
		EventTimer:Unlock()
		AshranBuddy.db.profile.locked = false
	end
	if (input == "lock") then
		EventTimer:Lock()
		AshranBuddy.db.profile.locked = true
	end
end

-- Inherited
function EventTimer:OnCreateOptions()
	AshranBuddy.options.args.eventtimer = {
		name = "Event Timer",
		type = "group",
		args = {
			locked = {
				name = "Lock Event Timer",
				desc = "Locks and unlocks the event timer frame.",
				type = "toggle",
				set = function(info, val)
					AshranBuddy.db.profile.locked = val
					if (AshranBuddy.db.profile.locked) then
						EventTimer:Lock()
					else
						EventTimer:Unlock()
					end
				end,
				get = function(info)
					return AshranBuddy.db.profile.locked
				end,
				order = 0
			},
			showTimer = {
				name = "Show Numbered Timer",
				desc = "Displays numbered timer on the right of the countdown bar.",
				type = "toggle",
				set = function(info, val)
					AshranBuddy.db.profile.showTimer = val
					if (AshranBuddy.db.profile.showTimer) then
						for i = 1, #self.statusBars do
							local statusBar = self.statusBars[i]
							EventTimer.statusBars[i].eventTimeFontString:Show()
						end
					else
						for i = 1, #self.statusBars do
							local statusBar = self.statusBars[i]
							EventTimer.statusBars[i].eventTimeFontString:Hide()
						end
					end
				end,
				get = function(info)
					return AshranBuddy.db.profile.showTimer
				end,
				order = 1
			},
		},
	}
end

--|c00FFFF00|Hspell:178019|h[Event: Collect Empowered Ore]|h|r begins at the Molten Quarry in 3 minutes.